/*
 ***** INCLUDE HEADERS *****
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>


/*
 ***** DEFINE RITARDI *****
 */

#define DELAY_PRE_COMPUTE                  12345
#define DELAY_PRE_COMPUTE2                 12345
#define DELAY_SHOW_OTHER_MOVE             500000
#define DELAY_THREAD_WAIT_MOVE             23256
#define DELAY_THREAD_WAIT_MOVE2            23256
#define DELAY_THREAD_WAIT_MOVE3            23256
#define DELAY_THREAD_BETWEEN_MOVE          63231
#define DELAY_THREAD_BETWEEN_PLAYERS      131313
#define DELAY_THREAD2_BETWEEN_STATES       13777
#define DELAY_THREAD2_BETWEEN_STATES2      88888
#define DELAY_THREAD2_BETWEEN_STATES_SLOW 500000

/*
 ***** DEFINE COLORI *****
 */

#define BLACK 0
#define RED 1
#define GREEN 2
#define BLUE 3
#define YELLOW 4
#define ORANGE 5
#define GOLD 6
#define SEA_GREEN 7
#define AQUAMARINE 8
#define NAVY_BLUE 9
#define LIGHT_TAN 10
#define LEMON 11
#define PURPLE 12
#define LIGHT_GREY 13
#define MID_GREY 14
#define DARK_GREY 15
#define RED_2 16
#define NAVY_BLUE_2 17
#define NAVY_BLUE_DARK 18
#define RED_DARK 19
#define WHITE 20


/*
 ***** DICHIARAZIONE FUNZIONI *****
 */
void gpk_game_reset(GtkButton *button);
void gpk_dump_chessboard(GtkButton *button);
void gpk_dump_valid_moves(void);
void gpk_dump_best_moves(void);
void gpk_update_chessboard(GtkButton *button);
void gpk_update_stat_num(GtkButton *button, int num, int val);
void gpk_update_stat_str(GtkButton *button, int num, char *str);
void gpk_update_stats(GtkButton *button);
void gpk_actor_game(int x, int y, GtkWidget *button);
void gpk_set_checkboard(GtkButton *button, int x, int y, gchar val, int colore);
void gpk_update_num_chips(int who, gchar type);
void gpk_check_gameover(GtkButton *button);
void gpk_backup_colors(void);
void gpk_alternate_colors(void);
void gpk_recovery_colors(void);
void gpk_automata(void);
void gpk_do_arrocco(int who);
int gpk_find_chips(int actual_player);

void gpk_insert_best_move(int x, int y);
int gpk_compute_best_moves(int x, int y);
void gpk_insert_valid_move(int x, int y);
int gpk_compute_valid_moves(int x, int y);
int gpk_check_this_move(int x, int y);

void gpk_thread(void *arg); // ----- thread di gioco automatico -----
void gpk_thread2(void *arg); // ----- thread di disegno gui -----

void gpk_draw_position(GtkWidget *here);
void gpk_set_color(GtkWidget *here, GdkGC *gc, gint name);


/*
 ***** VARIABILI DI GIOCO *****
 */
int ct;           // Pedina guantata dal robot (colore)
gchar tt;         // Pedina guantata dal robot (tipo)
gchar gt[8][12];  // Matrice pedine sulla scacchiera strategica (8x8 + puntatori esterni x arrocco)
gchar gt1[8][12]; // Matrice pedine sulla scacchiera realtime (8x8 + puntatori esterni x arrocco)
gchar dt[2][16]; // Matrice pedine sul deposito (per ogni giocatore)
int gc[8][12];    // Matrice colore pedine sulla scacchiera strategica (8x8 + puntatori esterni x arrocco)
int gc1[8][12];   // Matrice colore pedine sulla scacchiera realtime (8x8 + puntatori esterni x arrocco)
int gc2[8][12];   // Matrice colore pedine sulla scacchiera temporanea (x suggerimenti mosse)
int gn1[8];       // Contatore pedine PLAYER 1
int gn2[8];       // Contatore pedine PLAYER 2
int vittorie[2];  // Contatore vittorie
int mangiate[2];  // Contatore pedine mangiate

int stato=0;         // Macchina a Stati del gioco
int mossa[2];        // Memoria mossa
int num_mosse=0;     // Numero di mosse nella partita attuale
int actual_player=1; // Giocatore di mano
int arrocco_flag[2]; // E' possibile l'arrocco (k e t non mossi) ?
int arrocco_started; // E' da avviare l'arrocco (attendere richiesta) ?

int best_move[28][2];  // Mosse buone per la posizione attuale
int valid_move[28][2]; // Mosse valide per la posizione attuale
int valid_chip[28][2]; // Mosse valide per la posizione attuale
int best_move_cnt=0;   // Numero di mosse buone nella mano attuale
int valid_move_cnt=0;  // Numero di mosse valide nella mano attuale
int valid_chip_cnt=0;  // Numero di pedine sulla scacchiera

int stato_r=0;        // Macchina a Stati del robot
int x1,y1,x2,y2;      // Posizione start e stop robot (in coordinate stepper)
int xx,yy,zz;         // Posizione attuale del robot (in coordinate stepper)
int xo=9;             // Offset posizione spigolo scacchiera sulle X
int yo=7;             // Offset posizione spigolo scacchiera sulle Y
int zo=7;             // Offset per raggiungere la pedina in altezza
int xd[2];            // Prima posizione libera deposito mangiate (asse x)
int yd[2];            // Prima posizione libera deposito mangiate (asse y)

int thread_started=0;  // Il Thread di gioco e' avviato ?
int thread2_started=0; // Il Thread di gioco e' avviato ?
pthread_t thd;         // PThread Structures

static GdkPixmap *pixmap=NULL; // Area dove disegnare gli assi del robot


/*
 ***** DEFINIZIONI FUNZIONI *****
 */


/* ***** ASSEGNA UN COLORE A UN GRAPHIC CONTEXT ***** */
void gpk_set_color(GtkWidget *here, GdkGC *gc2, gint name)
{
	GdkColor colour;
	
	switch (name) {
	case BLACK:
		colour.red = 0;
		colour.green = 0;
		colour.blue = 0;
	break;
	case RED:
		colour.red = 65535;
		colour.green = 0;
		colour.blue = 0;
	break;
	case GREEN:
		colour.red = 0;
		colour.green = 65535;
		colour.blue = 0;
	break;
	case BLUE:
		colour.red = 0;
		colour.green = 0;
		colour.blue = 65535;
	break;
	case YELLOW:
		colour.red = 65535;
		colour.green = 65535;
		colour.blue = 0;
	break;
	case ORANGE:
	case GOLD:
		colour.red = 50000;
		colour.green = 50000;
		colour.blue = 10000;
	break;
	case SEA_GREEN:
		colour.red = 11150;
		colour.green = 55705;
		colour.blue = 40632;
	break;
	case AQUAMARINE:
		colour.red = 10000;
		colour.green = 65535;
		colour.blue = 50000;
	break;
	case NAVY_BLUE:
		colour.red = 0;
		colour.green = 32000;
		colour.blue = 65535;
	break;
	case LIGHT_TAN:
		colour.red = 65535;
		colour.green = 65535;
		colour.blue = 53739;
	break;
	case LEMON:
		colour.red = 64890;
		colour.green = 63569;
		colour.blue = 29491;
	break;
	case PURPLE:
		colour.red = 64000;
		colour.green = 30000;
		colour.blue = 64000;
	break;
	case LIGHT_GREY:
		colour.red = 58000;
		colour.green = 58000;
		colour.blue = 58000;
	break;
	case MID_GREY:
		colour.red = 32000;
		colour.green = 32000;
		colour.blue = 32000;
	break;
	case DARK_GREY:
		colour.red = 16000;
		colour.green = 16000;
		colour.blue = 16000;
	break;
	case RED_2:
		colour.red = 65535;
		colour.green = 32000;
		colour.blue = 32000;
	break;
	case NAVY_BLUE_2:
		colour.red = 22000;
		colour.green = 48000;
		colour.blue = 65535;
	break;
	case NAVY_BLUE_DARK:
		colour.red = 0;
		colour.green = 16000;
		colour.blue = 32000;
	break;
	case RED_DARK:
		colour.red = 48000;
		colour.green = 0;
		colour.blue = 0;
	break;
	case WHITE:
	default:
		colour.red = 65535;
		colour.green = 65535;
		colour.blue = 65535;
	break;
	}

	if (!gdk_color_alloc(gtk_widget_get_colormap(here), &colour)) g_print("Warning: colour assignment failed.\n");
	gdk_gc_set_foreground(gc2, &colour);
}


/* ***** DISEGNA SFONDO E SCACCHIERA NELLA DRAWING-AREA ***** */
void gpk_draw_position(GtkWidget *here) {

	// ---- local vars -----
	int i,j;
	int chip_width;
	GtkWidget *widget=lookup_widget(GTK_WIDGET(here), "drawingarea1");
	GdkGC *gc0=widget->style->black_gc;

	// ----- MALLOC FRAMEBUFFER -----
	if (pixmap) gdk_pixmap_unref(pixmap);
	pixmap = gdk_pixmap_new(widget->window, widget->allocation.width, widget->allocation.height, -1);
	
	// ----- CLEAR BACKBGROUND + CHECKBOARD BACKGROUND + DEPOTS BACKGROUND -----
	gpk_set_color(widget, gc0, WHITE);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 0, 0, widget->allocation.width, widget->allocation.height);
	gpk_set_color(widget, gc0, LIGHT_GREY);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 100, 80, 80, 80);
	gpk_set_color(widget, gc0, LIGHT_GREY);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 100, 40, 80, 20);
	gpk_set_color(widget, gc0, LIGHT_GREY);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 100, 180, 80, 20);

	// ----- DRAW CHECKBOARD -----
	gpk_set_color(widget, gc0, BLACK);
	for (j=0;j<4;j++) {
		for (i=100;i<180;i+=20) {
			gdk_draw_rectangle(pixmap, gc0, TRUE, i,    (80+(j*20)), 10, 10);
			gdk_draw_rectangle(pixmap, gc0, TRUE, i+10, (90+(j*20)), 10, 10);
		}
	}

	// ----- DRAW CHIP'S DEPOTS -----
	gpk_set_color(widget, gc0, NAVY_BLUE_DARK);
	for (i=100;i<180;i+=20) {
		gdk_draw_rectangle(pixmap, gc0, TRUE, i,    40, 10, 10);
		gdk_draw_rectangle(pixmap, gc0, TRUE, i+10, 50, 10, 10);
	}
	gpk_set_color(widget, gc0, RED_DARK);
	for (i=100;i<180;i+=20) {
		gdk_draw_rectangle(pixmap, gc0, TRUE, i,    180, 10, 10);
		gdk_draw_rectangle(pixmap, gc0, TRUE, i+10, 190, 10, 10);
	}

	// ----- DRAW CHIPS IN DEPOTS -----
	gpk_set_color(widget, gc0, BLACK);
	for (i=0;i<16;i++) {
		if (dt[0][i]!=' ') {
			chip_width=3;
			gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (45-chip_width), chip_width*2, chip_width*2);
			/*
			chip_width=3;
			if (dt[0][i]=='K') chip_width=5;
			if (dt[0][i]=='Q') chip_width=5;
			if (dt[0][i]=='p') chip_width=2;
			gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (45-chip_width), chip_width*2, chip_width*2);
			if (dt[0][i]=='K') chip_width=3;
			if (dt[0][i]=='Q') chip_width=2;
			if ((dt[0][i]=='K')||(dt[0][i]=='Q')) {
				gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (85-chip_width)+(j*10),
							chip_width*2, chip_width*2);
			}
			*/
		}
		if (dt[1][i]!=' ') {
			chip_width=3;
			gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (185-chip_width), chip_width*2, chip_width*2);
			/*
			chip_width=3;
			if (dt[1][i]=='K') chip_width=5;
			if (dt[1][i]=='Q') chip_width=5;
			if (dt[1][i]=='p') chip_width=2;
			gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (185-chip_width), chip_width*2, chip_width*2);
			if (dt[1][i]=='K') chip_width=3;
			if (dt[1][i]=='Q') chip_width=2;
			if (dt[1][i]=='K')||(dt[1][i]=='Q')) {
				gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (85-chip_width)+(j*10),
							chip_width*2, chip_width*2);
			}
			*/
		}
	}
	
	// ----- DISEGNA PEDINE -----
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			if (gt1[i][j]!=' ') {
				chip_width=3;
				if (gc1[i][j]==2) {
					gpk_set_color(widget, gc0, RED);
				} else {
					gpk_set_color(widget, gc0, NAVY_BLUE);
				}
				if (gt1[i][j]=='K') chip_width=5;
				if (gt1[i][j]=='Q') chip_width=5;
				if (gt1[i][j]=='p') chip_width=2;
				gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (85-chip_width)+(j*10), chip_width*2, chip_width*2);

				if (gc1[i][j]==2) {
					gpk_set_color(widget, gc0, RED_2);
				} else {
					gpk_set_color(widget, gc0, NAVY_BLUE_2);
				}
				if (gt1[i][j]=='K') chip_width=3;
				if (gt1[i][j]=='Q') chip_width=2;
				if ((gt1[i][j]=='K')||(gt1[i][j]=='Q')) {
					gdk_draw_rectangle(pixmap, gc0, TRUE, (105-chip_width)+i*10, (85-chip_width)+(j*10),
                                                           chip_width*2, chip_width*2);
				}
			}
		}
	}
	//gdk_draw_line(pixmap, widget->style->black_gc, 15+xx, 8, 15+xx, 245);
	//gdk_draw_line(pixmap, widget->style->black_gc, 10, 15+yy, 253, 15+yy);
	//gdk_draw_line(pixmap, widget->style->black_gc, 100+xx, 8, 100+xx, 245);
	//gdk_draw_line(pixmap, widget->style->black_gc, 10, 80+yy, 253, 80+yy);

	// ----- DISEGNA ASSI X e Y ROBOT -----
	gpk_set_color(widget, gc0, BLACK);
	gdk_draw_line(pixmap, gc0, 15+xx*10, 8, 15+xx*10, 245);
	gdk_draw_line(pixmap, gc0, 10, 15+yy*10, 253, 15+yy*10);

	// ----- DISEGNA EFFETTO ELEVAZIONE ASSE Z -----
	gpk_set_color(widget, gc0, YELLOW);
	gdk_draw_rectangle(pixmap, gc0, TRUE, 15+xx*10-zz, 15+yy*10-zz, zz*2, zz*2);
	
	// ----- DISEGNA PEDINA CALAMITATA -----
	if (ct>0) {
		if (ct==1) gpk_set_color(widget, gc0, NAVY_BLUE);
		if (ct==2) gpk_set_color(widget, gc0, RED);
		gdk_draw_rectangle(pixmap, gc0, TRUE, 13+xx*10, 13+yy*10, 5, 5);
	}

	// ----- AGGIORNA FRAMEBUFFER -----
	GdkRectangle update_rect;
	update_rect.x = 8;
	update_rect.y = 3;
	update_rect.width = 243;
	update_rect.height = 3;
	gtk_widget_draw (widget, &update_rect);
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)], pixmap,
			8, 5, 5, 0, widget->allocation.width, widget->allocation.height);
}


/* ***** AGGIORNA 1 STAT NUMERICA ***** */
void gpk_update_stat_num(GtkButton *button, int num, int val)
{
	gchar temp[2];
	gchar target[10];
	
	sprintf(temp, "%d", val);
	sprintf(target, "entry%d", num);
	GtkWidget* gwidget= lookup_widget(GTK_WIDGET(button), target);
	gtk_entry_set_text(GTK_ENTRY(gwidget), temp);
}


/* ***** AGGIORNA 1 STAT TESTUALE ***** */
void gpk_update_stat_str(GtkButton *button, int num, char *str)
{
	gchar target[10];
	
	sprintf(target, "entry%d", num);
	GtkWidget* gwidget= lookup_widget(GTK_WIDGET(button), target);
	gtk_entry_set_text(GTK_ENTRY(gwidget), str);
}


/* ***** AGGIORNA STATISTICHE ***** */
void gpk_update_stats(GtkButton *button)
{
	gpk_update_stat_num(button, 13, gn1[1]);
	gpk_update_stat_num(button, 14, gn1[2]);
	gpk_update_stat_num(button, 15, gn1[3]);
	gpk_update_stat_num(button, 16, gn1[4]);
	gpk_update_stat_num(button, 17, gn1[5]);

	gpk_update_stat_num(button, 18, gn2[1]);
	gpk_update_stat_num(button, 19, gn2[2]);
	gpk_update_stat_num(button, 20, gn2[3]);
	gpk_update_stat_num(button, 21, gn2[4]);
	gpk_update_stat_num(button, 22, gn2[5]);

	gpk_update_stat_num(button, 1, vittorie[0]);
	gpk_update_stat_num(button, 3, vittorie[1]);
	gpk_update_stat_num(button, 11, stato);
	gpk_update_stat_num(button, 7, actual_player);
	gpk_update_stat_num(button, 10, num_mosse);
}


/* ***** CONTA LE PEDINE ***** */
void gpk_update_num_chips(int who, gchar type) {
	if (who==2) {
		if (type=='K') gn2[0]--;
		if (type=='Q') gn2[1]--;
		if (type=='B') gn2[2]--;
		if (type=='H') gn2[3]--;
		if (type=='T') gn2[4]--;
		if (type=='p') gn2[5]--;
	} else {
		if (type=='K') gn1[0]--;
		if (type=='Q') gn1[1]--;
		if (type=='B') gn1[2]--;
		if (type=='H') gn1[3]--;
		if (type=='T') gn1[4]--;
		if (type=='p') gn1[5]--;
	}
}


/* ***** CONTA LE PEDINE ***** */
void gpk_check_gameover(GtkButton *button) {
	if (gn1[0]==0) {
		vittorie[0]++;
		stato=0;
		gpk_game_reset(button);
	}
	if (gn2[0]==0) {
		vittorie[1]++;
		stato=0;
		gpk_game_reset(button);
	}
}


/* ***** AUTOMATISMI DI GIOCO (promozione pedone) ***** */
void gpk_automata(void) {

	int i;

	for (i=0;i<8;i++) { // ----- PROMOZIONE PEDONE ROSSO -----
		if ((gt[i][0]=='p')&&(gc[i][0]==2)) {
			gt[i][0]='Q';
		}
	}
	for (i=0;i<8;i++) { // ----- PROMOZIONE PEDONE NERO -----
		if ((gt[i][7]=='p')&&(gc[i][7]==1)) {
			gt[i][7]='Q';
		}
	}
}


/* ***** ESEGUE L'ARROCCO ***** */
void gpk_do_arrocco(int who) {

	if (who==2) { // ----- ARROCCO ROSSO -----
		//gt[6][7]=gt[4][7];
		//gt[5][7]=gt[7][7];
		gt[4][7]=' ';
		gt[5][7]='T';
		gt[6][7]='K';
		gt[7][7]=' ';
		gc[4][7]=0;
		gc[5][7]=2;
		gc[6][7]=2;
		gc[7][7]=0;
	}
	if (who==1) { // ----- ARROCCO NERO -----
		//gt[6][0]=gt[4][0];
		//gt[5][0]=gt[7][0];
		gt[4][0]=' ';
		gt[5][0]='T';
		gt[6][0]='K';
		gt[7][0]=' ';
		gc[4][0]=0;
		gc[5][0]=1;
		gc[6][0]=1;
		gc[7][0]=0;
	}
}


/* ***** INSERISCE UNA NUOVA MOSSA POSSIBILE NELLA LISTA ***** */
void gpk_insert_valid_move(int x, int y) {
	valid_move[valid_move_cnt][0]=x;
	valid_move[valid_move_cnt][1]=y;
	valid_move_cnt++;
}


/* ***** INSERISCE UNA NUOVA BUONA MOSSA POSSIBILE NELLA LISTA ***** */
void gpk_insert_best_move(int x, int y) {
	best_move[best_move_cnt][0]=x;
	best_move[best_move_cnt][1]=y;
	best_move_cnt++;
}


/* ***** INSERISCE UNA NUOVA MOSSA POSSIBILE NELLA LISTA ***** */
int gpk_find_chips(int actual_player) {

	int i,j,cnt;

	cnt=0;
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			if ((gt[i][j]!=' ')&&(gc[i][j]==actual_player)) {
				valid_chip[cnt][0]=i;
				valid_chip[cnt][1]=j;
				cnt++;
			}
		}
	}
	return cnt;
}


/* ***** DUMP MATRICE MOSSE VALIDE ***** */
void gpk_dump_valid_moves(void) {

	int i;
	gchar temp[30];

	sprintf(temp, "Dump mosse valide: %d\n", valid_move_cnt);
	g_print (temp);
	for (i=0;i<valid_move_cnt;i++) {
		sprintf(temp, "[%d]-[%d]\n", valid_move[i][0], valid_move[i][1]);
		g_print (temp);
	}
	g_print ("\n");
}


/* ***** DUMP MATRICE MOSSE VALIDE ***** */
void gpk_dump_best_moves(void) {

	int i;
	gchar temp[30];

	sprintf(temp, "Dump mosse che mangiano: %d\n", best_move_cnt);
	g_print (temp);
	for (i=0;i<best_move_cnt;i++) {
		sprintf(temp, "[%d]-[%d]\n", best_move[i][0], best_move[i][1]);
		g_print (temp);
	}
	g_print ("\n");
}


/* ***** SALVA COLORI NELLA MATRICE TEMPORANEA ***** */
void gpk_backup_colors(void) {

	int i,j;

	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gc2[i][j]=gc[i][j];
		}
	}
}


/* ***** REIPRISTINA COLORI DALLA MATRICE TEMPORANEA ***** */
void gpk_recovery_colors(void) {

	int i,j;

	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gc[i][j]=gc2[i][j];
		}
	}
}


/* ***** REIPRISTINA COLORI DALLA MATRICE TEMPORANEA ***** */
void gpk_alternate_colors(void) {

	int i;

	for (i=0;i<valid_move_cnt;i++) {
		if (valid_move[i][1]==10) {        // ----- ARROCCO NERO -----
			gc[4][0]=4;
			gc[5][0]=4;
			gc[6][0]=4;
			gc[7][0]=4;
			arrocco_started=1;
		} else if (valid_move[i][1]==11) { // ----- ARROCCO ROSSO -----
			gc[4][7]=4;
			gc[5][7]=4;
			gc[6][7]=4;
			gc[7][7]=4;
			arrocco_started=1;
		} else {                           // ----- MOSSE NORMALI -----
			if (gc[valid_move[i][0]][valid_move[i][1]]==0) {
				gc[valid_move[i][0]][valid_move[i][1]]=3; // colora di giallo le mosse possibili
			} else {
				gc[valid_move[i][0]][valid_move[i][1]]=5; // colora di verde quando si mangia
			}
		}
	}
}


/* ***** CONTROLLA SE UNA MOSSA E' VALIDA ***** */
int gpk_check_this_move(int x, int y) {

	int i;
	
	for (i=0;i<28;i++) {
		if ((valid_move[i][0]==x)&&(valid_move[i][1]==y)) {
			return 1;
		}
	}
	return 0;
}


/* ***** DUMP MATRICE SCACCHIERA ***** */
void gpk_dump_chessboard(GtkButton *button) {
	
	int i,j;
	gchar temp[30];
	
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			sprintf(temp, "%d-", gt[i][j]);
			g_print (temp);
		}
		g_print ("\n");
	}
	g_print ("\n");
	sprintf(temp, "Stato: %d\n", stato);
	g_print (temp);
}


/* ***** AGGIORNA SCACCHIERA ***** */
void gpk_update_chessboard(GtkButton *button) {
	
	int i,j;
	gchar temp[30];
	
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gpk_set_checkboard(button, i, j, gt[i][j], gc[i][j]);
		}
	}
}


/* ***** RESET GIOCO ***** */
void gpk_game_reset(GtkButton *button) {

	// INIT
	int i,j;
	gtk_rc_parse("/home/gipoco/Projects/project1/src/joshua.rc");
	stato=0;
	stato_r==20;
	arrocco_flag[0]=1;
	arrocco_flag[1]=1;
	arrocco_started=0;
	actual_player=1;
	num_mosse=0;
	xx=0;
	yy=0;
	zz=0;
	tt=' ';
	ct=0;
	mangiate[0]=0;
	mangiate[1]=0;
	xd[0]=9;
	yd[0]=3;
	xd[1]=9;
	yd[1]=17;


	// RESET NUMERO PEDINE
	gn1[0]=1;
	gn1[1]=1;
	gn1[2]=2;
	gn1[3]=2;
	gn1[4]=2;
	gn1[5]=8;
	gn2[0]=1;
	gn2[1]=1;
	gn2[2]=2;
	gn2[3]=2;
	gn2[4]=2;
	gn2[5]=8;

	// RESET DEPOSITI PEDINE MANGIATE
	for (i=0;i<16;i++) {
		dt[0][i]=' ';
		dt[1][i]=' ';
	}

	// RESET POSIZIONI E COLORI
	for (i=0;i<8;i++) {
		for (j=0;j<8;j++) {
			gt[i][j]=' ';
			gt1[i][j]=' ';
			gc[i][j]=0;
			gc1[i][j]=0;
		}
	}		
	
	// METTI PEDINE IN MEMORIA
	gt[0][0]='T';
	gt[1][0]='H';
	gt[2][0]='B';
	gt[3][0]='Q';
 	gt[4][0]='K';
	gt[5][0]='B';
	gt[6][0]='H';
	gt[7][0]='T';
	for (i=0; i<8; i++) {
		gt[i][1]='p';
		gt[i][6]='p';
	}
	gt[0][7]='T';
	gt[1][7]='H';
	gt[2][7]='B';
	gt[3][7]='Q';
	gt[4][7]='K';
	gt[5][7]='B';
	gt[6][7]='H';
	gt[7][7]='T';

	gt1[0][0]='T';
	gt1[1][0]='H';
	gt1[2][0]='B';
	gt1[3][0]='Q';
 	gt1[4][0]='K';
	gt1[5][0]='B';
	gt1[6][0]='H';
	gt1[7][0]='T';
	for (i=0; i<8; i++) {
		gt1[i][1]='p';
		gt1[i][6]='p';
	}
	gt1[0][7]='T';
	gt1[1][7]='H';
	gt1[2][7]='B';
	gt1[3][7]='Q';
	gt1[4][7]='K';
	gt1[5][7]='B';
	gt1[6][7]='H';
	gt1[7][7]='T';


	// METTI COLORI IN MEMORIA
	for (i=0; i<8; i++) {
		gc[i][0]=1;
		gc[i][1]=1;
		gc[i][6]=2;
		gc[i][7]=2;
		gc1[i][0]=1;
		gc1[i][1]=1;
		gc1[i][6]=2;
		gc1[i][7]=2;
	}
	
	// DISEGNA SCACCHIERA E ASSI ROBOT
	gpk_draw_position(button);
	
	// START ROBOT THREAD	
	if (thread2_started==0) {
		pthread_create(&thd, NULL, gpk_thread2, button);
		g_print ("Thread2 started now....\n");
		thread2_started++;
	}

	// STAMPA MESSAGGIO
	gpk_update_stat_str(button, 24, "Choose num players");
}


/* ***** CALCOLA E COLORA MOSSE VALIDE DA UNA POSIZIONE DATA ***** */
int gpk_compute_valid_moves(int x, int y) {

	int i,j,flag;
	int my_color=((gc[x][y]==1)?1:2);
	int his_color=((gc[x][y]==1)?2:1);

	valid_move_cnt=0;

	for (i=0;i<28;i++) {
		valid_move[i][0]=0;
		valid_move[i][1]=0;
	}

	if (gt[x][y]=='K') { // ------------------- R E -----------------------
		
		// --- normali ---
		if ((x<7)&&((gc[x+1][y]==his_color)||(gt[x+1][y]==' '))) gpk_insert_valid_move(x+1, y);
		if ((x>0)&&((gc[x-1][y]==his_color)||(gt[x-1][y]==' '))) gpk_insert_valid_move(x-1, y);
		if ((y>0)&&((gc[x][y-1]==his_color)||(gt[x][y-1]==' '))) gpk_insert_valid_move(x, y-1);
		if ((y<7)&&((gc[x][y+1]==his_color)||(gt[x][y+1]==' '))) gpk_insert_valid_move(x, y+1);

		// --- diagonali ---
		if ((x<7)&&(y<7)&&((gc[x+1][y+1]==his_color)||(gt[x+1][y+1]==' '))) gpk_insert_valid_move(x+1, y+1);
		if ((x>0)&&(y>0)&&((gc[x-1][y-1]==his_color)||(gt[x-1][y-1]==' '))) gpk_insert_valid_move(x-1, y-1);
		if ((x<7)&&(y>0)&&((gc[x+1][y-1]==his_color)||(gt[x+1][y-1]==' '))) gpk_insert_valid_move(x+1, y-1);
		if ((x>0)&&(y<7)&&((gc[x-1][y+1]==his_color)||(gt[x-1][y+1]==' '))) gpk_insert_valid_move(x-1, y+1);

		// --- arrocco NERO ---
		if ((arrocco_flag[0])&&(gt[5][0]==' ')&&(gt[6][0]==' ')) gpk_insert_valid_move(6, 10);

		// --- arrocco ROSSO ---
		if ((arrocco_flag[1])&&(gt[5][7]==' ')&&(gt[6][7]==' ')) gpk_insert_valid_move(6, 11);
	}

	if (gt[x][y]=='H') { // ------------------- C A V A L L O -----------------------
		
		if ((x<7)&&(y<6)&&((gc[x+1][y+2]==his_color)||(gt[x+1][y+2]==' '))) gpk_insert_valid_move(x+1, y+2);
		if ((x<6)&&(y<7)&&((gc[x+2][y+1]==his_color)||(gt[x+2][y+1]==' '))) gpk_insert_valid_move(x+2, y+1);
		if ((x>0)&&(y>1)&&((gc[x-1][y-2]==his_color)||(gt[x-1][y-2]==' '))) gpk_insert_valid_move(x-1, y-2);
		if ((x>1)&&(y>0)&&((gc[x-2][y-1]==his_color)||(gt[x-2][y-1]==' '))) gpk_insert_valid_move(x-2, y-1);
		if ((x>0)&&(y<6)&&((gc[x-1][y+2]==his_color)||(gt[x-1][y+2]==' '))) gpk_insert_valid_move(x-1, y+2);
		if ((x<6)&&(y>0)&&((gc[x+2][y-1]==his_color)||(gt[x+2][y-1]==' '))) gpk_insert_valid_move(x+2, y-1);
		if ((x<7)&&(y>1)&&((gc[x+1][y-2]==his_color)||(gt[x+1][y-2]==' '))) gpk_insert_valid_move(x+1, y-2);
		if ((x>1)&&(y<7)&&((gc[x-2][y+1]==his_color)||(gt[x-2][y+1]==' '))) gpk_insert_valid_move(x-2, y+1);
	}

	if (gt[x][y]=='p') { // ------------------- P E D O N E -----------------------
		
		if (gc[x][y]==1) { // ----- NERO -----
			if ((y<7)&&(gt[x][y+1]==' '))        gpk_insert_valid_move(x, y+1);   // scende
			if ((y==1)&&(gt[x][3]==' '))         gpk_insert_valid_move(x, 3);     // subito salta 2
			if ((x<7)&&(y<7)&&(gc[x+1][y+1]==2)) gpk_insert_valid_move(x+1, y+1); // mangia a destra
			if ((x>0)&&(y<7)&&(gc[x-1][y+1]==2)) gpk_insert_valid_move(x-1, y+1); // mangia a sinistra
		}
		if (gc[x][y]==2) { // ----- ROSSO -----
			if ((y>0)&&(gt[x][y-1]==' '))        gpk_insert_valid_move(x, y-1);   // sale
			if ((y==6)&&(gt[x][4]==' '))         gpk_insert_valid_move(x, 4);     // subito salta 2
			if ((x<7)&&(y>0)&&(gc[x+1][y-1]==1)) gpk_insert_valid_move(x+1, y-1); // mangia a destra
			if ((x>0)&&(y>0)&&(gc[x-1][y-1]==1)) gpk_insert_valid_move(x-1, y-1); // mangia a sinistra
		}
	}

	if ((gt[x][y]=='T')||(gt[x][y]=='Q')) { // ------------------- T O R R E  /  R E G I N A -----------------------

		// -- scende --
		flag=0; i=y+1;
		while ((flag==0)&&(i<8)) {
			if (gt[x][i]==' ') gpk_insert_valid_move(x, i);
			if (gt[x][i]!=' ') {
				if (gc[x][i]==his_color) gpk_insert_valid_move(x, i);
				flag++;
			}
			i++;
		}

		// -- sale --
		flag=0; i=y-1;
		while ((flag==0)&&(i>=0)) {
			if (gt[x][i]==' ') gpk_insert_valid_move(x, i);
			if (gt[x][i]!=' ') {
				if (gc[x][i]==his_color) gpk_insert_valid_move(x, i);
				flag++;
			}
			i--;
		}

		// -- slide DX --
		flag=0; i=x+1;
		while ((flag==0)&&(i<8)) {
			if (gt[i][y]==' ') gpk_insert_valid_move(i, y);
			if (gt[i][y]!=' ') {
				if (gc[i][y]==his_color) gpk_insert_valid_move(i, y);
				flag++;
			}
			i++;
		}

		// -- slide SX --
		flag=0; i=x-1;
		while ((flag==0)&&(i>=0)) {
			if (gt[i][y]==' ') gpk_insert_valid_move(i, y);
			if (gt[i][y]!=' ') {
				if (gc[i][y]==his_color) gpk_insert_valid_move(i, y);
				flag++;
			}
			i--;
		}
	}

	if ((gt[x][y]=='B')||(gt[x][y]=='Q')) { // ------------------- A L F I E R E  /  R E G I N A -----------------------

		// -- DOWN DX --
		flag=0; i=x+1; j=y+1;
		while ((flag==0)&&(i<8)&&(j<8)) {
			if (gt[i][j]==' ') gpk_insert_valid_move(i, j);
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_valid_move(i, j);
				flag++;
			}
			i++; j++;
		}

		// -- DOWN SX --
		flag=0; i=x-1; j=y+1;
		while ((flag==0)&&(i>=0)&&(j<8)) {
			if (gt[i][j]==' ') gpk_insert_valid_move(i, j);
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_valid_move(i, j);
				flag++;
			}
			i--; j++;
		}

		// -- UP SX --
		flag=0; i=x-1; j=y-1;
		while ((flag==0)&&(i>=0)&&(j>=0)) {
			if (gt[i][j]==' ') gpk_insert_valid_move(i, j);
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_valid_move(i, j);
				flag++;
			}
			i--; j--;
		}

		// -- UP DX --
		flag=0; i=x+1; j=y-1;
		while ((flag==0)&&(i<8)&&(j>=0)) {
			if (gt[i][j]==' ') gpk_insert_valid_move(i, j);
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_valid_move(i, j);
				flag++;
			}
			i++; j--;
		}
	}

	return valid_move_cnt;
}


/* ***** CERCA MOSSE MIGLIORI DA UNA POSIZIONE DATA ***** */
int gpk_compute_best_moves(int x, int y) {

	int i,j,flag;
	int my_color=((gc[x][y]==1)?1:2);
	int his_color=((gc[x][y]==1)?2:1);

	best_move_cnt=0;

	for (i=0;i<28;i++) {
		best_move[i][0]=0;
		best_move[i][1]=0;
	}

	if (gt[x][y]=='K') { // ------------------- R E -----------------------
		
		// --- normali ---
		if ((x<7)&&(gc[x+1][y]==his_color)) gpk_insert_best_move(x+1, y);
		if ((x>0)&&(gc[x-1][y]==his_color)) gpk_insert_best_move(x-1, y);
		if ((y>0)&&(gc[x][y-1]==his_color)) gpk_insert_best_move(x, y-1);
		if ((y<7)&&(gc[x][y+1]==his_color)) gpk_insert_best_move(x, y+1);

		// --- diagonali ---
		if ((x<7)&&(y<7)&&(gc[x+1][y+1]==his_color)) gpk_insert_best_move(x+1, y+1);
		if ((x>0)&&(y>0)&&(gc[x-1][y-1]==his_color)) gpk_insert_best_move(x-1, y-1);
		if ((x<7)&&(y>0)&&(gc[x+1][y-1]==his_color)) gpk_insert_best_move(x+1, y-1);
		if ((x>0)&&(y<7)&&(gc[x-1][y+1]==his_color)) gpk_insert_best_move(x-1, y+1);
	}

	if (gt[x][y]=='H') { // ------------------- C A V A L L O -----------------------
		
		if ((x<7)&&(y<6)&&(gc[x+1][y+2]==his_color)) gpk_insert_best_move(x+1, y+2);
		if ((x<6)&&(y<7)&&(gc[x+2][y+1]==his_color)) gpk_insert_best_move(x+2, y+1);
		if ((x>0)&&(y>1)&&(gc[x-1][y-2]==his_color)) gpk_insert_best_move(x-1, y-2);
		if ((x>1)&&(y>0)&&(gc[x-2][y-1]==his_color)) gpk_insert_best_move(x-2, y-1);
		if ((x>0)&&(y<6)&&(gc[x-1][y+2]==his_color)) gpk_insert_best_move(x-1, y+2);
		if ((x<6)&&(y>0)&&(gc[x+2][y-1]==his_color)) gpk_insert_best_move(x+2, y-1);
		if ((x<7)&&(y>1)&&(gc[x+1][y-2]==his_color)) gpk_insert_best_move(x+1, y-2);
		if ((x>1)&&(y<7)&&(gc[x-2][y+1]==his_color)) gpk_insert_best_move(x-2, y+1);
	}

	if (gt[x][y]=='p') { // ------------------- P E D O N E -----------------------
		
		if (gc[x][y]==1) { // ----- NERO -----
			if ((x<7)&&(y<7)&&(gc[x+1][y+1]==2)) gpk_insert_best_move(x+1, y+1); // mangia a destra
			if ((x>0)&&(y<7)&&(gc[x-1][y+1]==2)) gpk_insert_best_move(x-1, y+1); // mangia a sinistra
		}
		if (gc[x][y]==2) { // ----- ROSSO -----
			if ((x<7)&&(y>0)&&(gc[x+1][y-1]==1)) gpk_insert_best_move(x+1, y-1); // mangia a destra
			if ((x>0)&&(y>0)&&(gc[x-1][y-1]==1)) gpk_insert_best_move(x-1, y-1); // mangia a sinistra
		}
	}

	if ((gt[x][y]=='T')||(gt[x][y]=='Q')) { // ------------------- T O R R E  /  R E G I N A -----------------------

		// -- scende --
		flag=0; i=y+1;
		while ((flag==0)&&(i<8)) {
			if (gt[x][i]!=' ') {
				if (gc[x][i]==his_color) gpk_insert_best_move(x, i);
				flag++;
			}
			i++;
		}

		// -- sale --
		flag=0; i=y-1;
		while ((flag==0)&&(i>=0)) {
			if (gt[x][i]!=' ') {
				if (gc[x][i]==his_color) gpk_insert_best_move(x, i);
				flag++;
			}
			i--;
		}

		// -- slide DX --
		flag=0; i=x+1;
		while ((flag==0)&&(i<8)) {
			if (gt[i][y]!=' ') {
				if (gc[i][y]==his_color) gpk_insert_best_move(i, y);
				flag++;
			}
			i++;
		}

		// -- slide SX --
		flag=0; i=x-1;
		while ((flag==0)&&(i>=0)) {
			if (gt[i][y]!=' ') {
				if (gc[i][y]==his_color) gpk_insert_best_move(i, y);
				flag++;
			}
			i--;
		}
	}

	if ((gt[x][y]=='B')||(gt[x][y]=='Q')) { // ------------------- A L F I E R E  /  R E G I N A -----------------------

		// -- DOWN DX --
		flag=0; i=x+1; j=y+1;
		while ((flag==0)&&(i<8)&&(j<8)) {
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_best_move(i, j);
				flag++;
			}
			i++; j++;
		}

		// -- DOWN SX --
		flag=0; i=x-1; j=y+1;
		while ((flag==0)&&(i>=0)&&(j<8)) {
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_best_move(i, j);
				flag++;
			}
			i--; j++;
		}

		// -- UP SX --
		flag=0; i=x-1; j=y-1;
		while ((flag==0)&&(i>=0)&&(j>=0)) {
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_best_move(i, j);
				flag++;
			}
			i--; j--;
		}

		// -- UP DX --
		flag=0; i=x+1; j=y-1;
		while ((flag==0)&&(i<8)&&(j>=0)) {
			if (gt[i][j]!=' ') {
				if (gc[i][j]==his_color) gpk_insert_best_move(i, j);
				flag++;
			}
			i++; j--;
		}
	}

	return valid_move_cnt;
}


/* ***** CONTROLLA ED EFFETTUA MOSSA ***** */
void gpk_actor_game(int x, int y, GtkWidget *button) {
	
	int changed=0;
	
	if ((stato==0)||(stato==10)||(stato==20)||(stato==22)) {                      // PRIMA GUANTO LA PEDINA.....
		while (stato_r>0) usleep(DELAY_PRE_COMPUTE); // aspetta se ci sono movimenti del robot in corso
		if (stato==10) gpk_compute_best_moves(x,y);  // Se gioca da solo cerca mosse furbe....
		if (gpk_compute_valid_moves(x,y)>0) {        // ....e cque tutte le mosse valide
			//gpk_dump_valid_moves();
			mossa[0]=x;
			mossa[1]=y;
			gpk_backup_colors();
			gpk_alternate_colors();
			stato++;
			changed++;
		}
	}
	if (((stato==1)||(stato==11)||(stato==21)||(stato==23))&&(changed==0)) {      // POI LA RIMETTO SULLA SCACCHIERA
		while (stato_r>0) usleep(DELAY_PRE_COMPUTE2); // aspetta se ci sono movimenti del robot in corso
		if (stato==23) usleep(DELAY_SHOW_OTHER_MOVE);                // delay per far vedere la mossa avversaria
		gpk_recovery_colors();
		if (arrocco_started) {
			if ((x==6)&&(y==0)) y=10;
			if ((x==6)&&(y==7)) y=11;
			arrocco_started=0;
		}
		if (gpk_check_this_move(x,y)) {
			if (y>9) {                            // se e' richiesto l'arrocco fa la procedura speciale
				gpk_do_arrocco(gc[mossa[0]][mossa[1]]);
			} else {                              // altrimenti esegue una mossa normale
				if (gt[mossa[0]][mossa[1]]=='K') arrocco_flag[(gc[mossa[0]][mossa[1]]==2)?1:0]=0;
				if (gc[x][y]>0) gpk_update_num_chips((gc[mossa[0]][mossa[1]]), gt[x][y]);
				// ----- aggiorna l'array -----
				gt[x][y]=gt[mossa[0]][mossa[1]];
				gc[x][y]=gc[mossa[0]][mossa[1]];
				gt[mossa[0]][mossa[1]]=' ';
				gc[mossa[0]][mossa[1]]=0;
				// ----- muove il robot -----
				x1=mossa[0];
				y1=mossa[1];
				x2=x;
				y2=y;
				stato_r=5;
			}
		} else {
			if (stato==21) stato--;
			actual_player++;
		}
		if (stato==20) {
			//stato++;
		} else if (stato==21) {
			stato++;
		} else if (stato==23) {
			stato=20;
		} else {
			stato--;
		}
		gpk_automata();
		num_mosse++;
		actual_player++;
		if (actual_player>2) actual_player=1;
	}
	gpk_draw_position(button);
	gpk_update_stats((GtkButton *)button);
	gpk_check_gameover((GtkButton *)button);
	gpk_update_chessboard((GtkButton *)button);
	//gtk_widget_focus_out(button); // ----------------- Toglie la selezione BLU dalla scacchiera ------------------
}


/* ***** Wrapper scrittura carattere sulla scacchiera ***** */
void gpk_set_checkboard(GtkButton *button, int x, int y, gchar val, int colore) {
	
	gchar temp[2];
	gchar target[10];
	
	sprintf(temp, "%c", val);
	sprintf(target, "entry%d", (short)(26+x+(y*8)));
	GtkWidget* gwidget= lookup_widget(GTK_WIDGET(button), target);
	gtk_entry_set_text(GTK_ENTRY(gwidget), temp);
	if (colore==0) { gtk_widget_set_name (gwidget, "grigio"); } // ----- VUOTO -----
	if (colore==1) { gtk_widget_set_name (gwidget, "nero"); }   // ----- PLAYER 1 -----
	if (colore==2) { gtk_widget_set_name (gwidget, "rosso"); }  // ----- PLAYER 2 -----
	if (colore==3) { gtk_widget_set_name (gwidget, "giallo"); } // ----- MOSSA -----
	if (colore==4) { gtk_widget_set_name (gwidget, "viola"); }  // ----- MANGIA -----
	if (colore==5) { gtk_widget_set_name (gwidget, "verde"); }  // ----- SPECIAL -----
}


/* ***** RESET BUTTON***** */
void on_button65_clicked (GtkButton *button, gpointer user_data)
{
	vittorie[0]=0;
	vittorie[1]=0;
	gpk_game_reset(button);
	gpk_update_chessboard(button);
	gpk_update_stats(button);
	g_print ("Call Reset\n");
}


/* ***** 2 PLAYERS BUTTON***** */
void on_button66_clicked (GtkButton *button, gpointer user_data)
{
	gpk_update_stat_str(button, 24, "Play free chess");
	g_print ("Call 2 Players\n");
}


/* ***** 1 PLAYER BUTTON***** */
void on_button67_clicked (GtkButton *button, gpointer user_data)
{
	gpk_update_stat_str(button, 24, "You are black");
	g_print ("Call 1 Players\n");
	stato=20;
	if (thread_started==0) {
		pthread_create(&thd, NULL, gpk_thread, button);
		g_print ("Thread started now....\n");
		thread_started++;
	}
	g_print ("Thread informaed to start game....\n");
}


/* ***** 0 PLAYERS BUTTON ***** */
void on_button68_clicked (GtkButton *button, gpointer user_data)
{
	gpk_update_stat_str(button, 24, "Autoplay");
	g_print ("Call 0 Players\n");
	stato=10;
	if (thread_started==0) {
		pthread_create(&thd, NULL, gpk_thread, button);
		g_print ("Thread started now....\n");
		thread_started++;
	}
	g_print ("Thread informaed to start game....\n");
}


/* ***** AUTOPLAY THREAD ***** */
void gpk_thread(void *arg) {

	int x,y,r;
	GtkButton *button=(GtkButton *)arg;

	g_print ("Thread UP and RUNNING !!\n");
	while (1) {
		while ((stato==9)||(stato==10)||(stato==22)||(stato==23)) {
			g_print ("Hit....");
			valid_chip_cnt=gpk_find_chips(actual_player);
			if (valid_chip_cnt>0) {
				x=valid_chip[(int)(random()%valid_chip_cnt)][0];
				y=valid_chip[(int)(random()%valid_chip_cnt)][1];
				while (stato_r>0) usleep(DELAY_THREAD_WAIT_MOVE);  // OCCIO DEADLOCK !!
				gdk_threads_enter();
				gpk_actor_game(x, y, button);
				gdk_threads_leave();
				usleep(DELAY_THREAD_BETWEEN_MOVE);
				if (best_move_cnt>0) {                         // ----- cerca la prima mossa buona (o 1 a caso o la migliore) -----
					//gpk_dump_best_moves();
					/*
					r=(int)(random()%best_move_cnt);
					gdk_threads_enter();
					gpk_actor_game(best_move[r][0], best_move[r][1], button);
					gdk_threads_leave();
					usleep(100000);
					*/
					while (stato_r>0) usleep(DELAY_THREAD_WAIT_MOVE2);  // OCCIO DEADLOCK !!
					/*
					gchar temp[2];
					sprintf(temp, "Pedina: %c", gt[x][y]);
					g_print (temp);
					g_print ("\n");
					*/
					gdk_threads_enter();
					gpk_actor_game(best_move[0][0], best_move[0][1], button);
					gdk_threads_leave();
					g_print (" GOOD move found !!\n");
				} else if (valid_move_cnt>0) {                  // -------- se non la trova ne fa una a caso delle valide --------
					r=(int)(random()%valid_move_cnt);
					while (stato_r>0) usleep(DELAY_THREAD_WAIT_MOVE3);  // OCCIO DEADLOCK !!
					gdk_threads_enter();
					gpk_actor_game(valid_move[r][0], valid_move[r][1], button);
					gdk_threads_leave();
					g_print (" valid found\n");
				} else {
					g_print (" miss\n");
				}
			} else {
				g_print ("NO CHIPS (You would not be here !!)\n");
			}
		}
		usleep(DELAY_THREAD_BETWEEN_PLAYERS);
	}
	g_print ("THREAD END (You would not be here !!)\n");
}


/* ***** DRAW AREA / MOVE ROBOT THREAD ***** */
void gpk_thread2(void *arg) {
	
	int stato_old;                              // stato[1..49] ==> moove    stato[50..99] ==> mangia
	int player=0;
	gchar temp[35];
	GtkButton *here=(GtkButton *)arg;

	g_print ("Thread2 UP and RUNNING !!\n");
	while (1) {
		while (stato_r>0) {                               // ----- stato di quiete -----
			stato_old=stato_r;
			if (stato_old==5) {                       // ----- stato controllo mangiata -----
				if (gt1[x2][y2]==' ') {
					stato_r=7;
				} else {
					stato_r=50;
					player=((gc1[x1][y1]==1)?1:0);
				}
			}
			
			// ----- da qui in poi stati per lo spostamento -----
			
			if (stato_old==7) {                       // ----- stato di accelerazione -----
				if (xx<(x1+xo)) xx++;
				if (yy<(y1+yo)) yy++;
				if ((xx==(x1+xo))&&(yy==(y1+yo))) stato_r=10;
			}
			if (stato_old==10) {                      // ----- stato abbassamento su pedina -----
				if (zz<zo) {
					zz++;
				} else {
					stato_r=11;
				}
			}
			if (stato_old==11) {                      // ----- stato guanta pedina -----
				tt=gt1[x1][y1];
				ct=gc1[x1][y1];
				gt1[x1][y1]=' ';
				gc1[x1][y1]=0;
				stato_r=12;
			}
			if (stato_old==12) {                      // ----- stato porta su pedina -----
				if (zz>0) {
					zz--;
				} else {
					stato_r=15;
				}
			}
			if (stato_old==15) {                       // ----- stato sposta via pedina -----
				if (xx!=(x2+xo)) {
					if (xx<(x2+xo)) {
						xx++;
					} else {
						xx--;
					}
				}
				if (yy!=(y2+yo)) {
					if (yy<(y2+yo)) {
						yy++;
					} else {
						yy--;
					}
				}
				if ((xx==(x2+xo))&&(yy==(y2+yo))) stato_r=16;
			}
			if (stato_old==16) {                      // ----- stato abbassa pedina -----
				if (zz<zo) {
					zz++;
				} else {
					stato_r=18;
				}
			}
			if (stato_old==18) {                      // ----- stato rilascia pedina -----
				gt1[x2][y2]=tt;
				gc1[x2][y2]=ct;
				tt=' ';
				ct=0;
				stato_r=19;
			}
			if (stato_old==19) {                      // ----- stato risale -----
				if (zz>0) {
					zz--;
				} else {
					stato_r=20;
				}
			}
			if (stato_old==20) {                      // ----- stato di ritorno -----
				if (xx>0) xx--;
				if (yy>0) yy--;
				if ((xx==0)&&(yy==0)) stato_r=30; // ----- stato di fine -----
			}
			
			// ----- da qui in poi stati per la mangiata -----
			
			if (stato_old==50) {                       // ----- stato di accelerazione -----
				if (xx<(x2+xo)) xx++;
				if (yy<(y2+yo)) yy++;
				if ((xx==(x2+xo))&&(yy==(y2+yo))) stato_r=55;
			}
			if (stato_old==55) {                      // ----- stato abbassamento su pedina -----
				if (zz<zo) {
					zz++;
				} else {
					stato_r=60;
				}
			}
			if (stato_old==60) {                      // ----- stato guanta pedina -----
				tt=gt1[x2][y2];
				ct=gc1[x2][y2];
				gt1[x2][y2]=' ';
				gc1[x2][y2]=0;
				stato_r=65;
			}
			if (stato_old==65) {                      // ----- stato porta su pedina -----
				if (zz>0) {
					zz--;
				} else {
					stato_r=70;
				}
			}
			if (stato_old==70) {                       // ----- stato sposta via pedina -----
				if (xx!=(xd[player])) {
					if (xx<(xd[player])) {
						xx++;
					} else {
						xx--;
					}
				}
				if (yy!=(yd[player])) {
					if (yy<(yd[player])) {
						yy++;
					} else {
						yy--;
					}
				}
				if ((xx==(xd[player]))&&(yy==(yd[player]))) stato_r=75;
			}
			if (stato_old==75) {                      // ----- stato abbassa pedina -----
				if (zz<zo) {
					zz++;
				} else {
					stato_r=80;
				}
			}
			if (stato_old==80) {                      // ----- stato rilascia pedina -----
				dt[player][mangiate[player]]=tt;
				tt=' ';
				ct=0;
				stato_r=85;
			}
			if (stato_old==85) {                      // ----- stato risale -----
				if (zz>0) {
					zz--;
				} else {
					stato_r=90;
				}
			}
			if (stato_old==90) {                       // ----- stato va su source -----
				if (xx!=(x1+xo)) {
					if (xx<(x1+xo)) {
						xx++;
					} else {
						xx--;
					}
				}
				if (yy!=(y1+yo)) {
					if (yy<(y1+yo)) {
						yy++;
					} else {
						yy--;
					}
				}
				if ((xx==(x1+xo))&&(yy==(y1+yo))) stato_r=95;
			}
			if (stato_old==95) {                      // ----- stato abbassamento su pedina -----
				if (zz<zo) {
					zz++;
				} else {
					stato_r=100;
				}
			}
			if (stato_old==100) {                      // ----- stato guanta pedina -----
				tt=gt1[x1][y1];
				ct=gc1[x1][y1];
				gt1[x1][y1]=' ';
				gc1[x1][y1]=0;
				stato_r=105;
			}
			if (stato_old==105) {                      // ----- stato porta su pedina -----
				if (zz>0) {
					zz--;
				} else {
					stato_r=110;
				}
			}
			if (stato_old==110) {                       // ----- stato sposta via pedina -----
				if (xx!=(x2+xo)) {
					if (xx<(x2+xo)) {
						xx++;
					} else {
						xx--;
					}
				}
				if (yy!=(y2+yo)) {
					if (yy<(y2+yo)) {
						yy++;
					} else {
						yy--;
					}
				}
				if ((xx==(x2+xo))&&(yy==(y2+yo))) stato_r=115;
			}
			if (stato_old==115) {                      // ----- stato abbassa pedina -----
				if (zz<zo) {
					zz++;
				} else {
					stato_r=120;
				}
			}
			if (stato_old==120) {                      // ----- stato rilascia pedina -----
				gt1[x2][y2]=tt;
				gc1[x2][y2]=ct;
				tt=' ';
				ct=0;
				stato_r=125;
			}
			if (stato_old==125) {                      // ----- stato risale -----
				if (zz>0) {
					zz--;
				} else {
					stato_r=130;
				}
			}
			if (stato_old==130) {                      // ----- stato di ritorno -----
				if (xx>0) xx--;
				if (yy>0) yy--;
				if ((xx==0)&&(yy==0)) stato_r=135;
			}
			if (stato_old==135) {                      // ----- stato di update stats -----
				xd[player]++;
				if (xd[player]>17) {
					yd[player]++;
					xd[player]=9;
				}
				mangiate[player]++;
				stato_r=30;                        // ----- stato di fine -----
			}
			
			/* ***** AGGIORNAMENTO GUI ***** */
			gdk_threads_enter();
			gpk_draw_position(here);
			gdk_threads_leave();
			
			//sprintf(temp, "Stato_r[%d][%d]: %d\n", xx, yy, stato_r);
			//g_print (temp);
			/*
			if (stato_r>49) {
				usleep(DELAY_THREAD2_BETWEEN_STATES_SLOW);
			} else {
				usleep(DELAY_THREAD2_BETWEEN_STATES);
			}
			*/
			usleep(DELAY_THREAD2_BETWEEN_STATES);
			if (stato_r==30) stato_r=0;              // ----- stato di quiete -----
		}
		usleep(DELAY_THREAD2_BETWEEN_STATES2);
	}
	g_print ("THREAD END (You would not be here !!)\n");
}


/* ***** INTERCETTA CALLBACK SEGNALI ***** */
gboolean on_entry1_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { g_print ("Call GG\n"); return FALSE; }

gboolean on_entry26_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 0, widget); return FALSE; }
gboolean on_entry27_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 0, widget); return FALSE; }
gboolean on_entry28_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 0, widget); return FALSE; }
gboolean on_entry29_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 0, widget); return FALSE; }
gboolean on_entry30_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 0, widget); return FALSE; }
gboolean on_entry31_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(5, 0, widget); return FALSE; }
gboolean on_entry32_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(6, 0, widget); return FALSE; }
gboolean on_entry33_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(7, 0, widget); return FALSE; }

gboolean on_entry34_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(0, 1, widget); return FALSE; }
gboolean on_entry35_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(1, 1, widget); return FALSE; }
gboolean on_entry36_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(2, 1, widget); return FALSE; }
gboolean on_entry37_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(3, 1, widget); return FALSE; }
gboolean on_entry38_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(4, 1, widget); return FALSE; }
gboolean on_entry39_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(5, 1, widget); return FALSE; }
gboolean on_entry40_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(6, 1, widget); return FALSE; }
gboolean on_entry41_button_press_event (GtkWidget *widget, GdkEventButton  *event, gpointer user_data) { gpk_actor_game(7, 1, widget); return FALSE; }

gboolean on_entry42_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 2, widget); return FALSE; }
gboolean on_entry43_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 2, widget); return FALSE; }
gboolean on_entry44_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 2, widget); return FALSE; }
gboolean on_entry45_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 2, widget); return FALSE; }
gboolean on_entry46_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 2, widget); return FALSE; }
gboolean on_entry47_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(5, 2, widget); return FALSE; }
gboolean on_entry48_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(6, 2, widget); return FALSE; }
gboolean on_entry49_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(7, 2, widget); return FALSE; }

gboolean on_entry50_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 3, widget); return FALSE; }
gboolean on_entry51_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 3, widget); return FALSE; }
gboolean on_entry52_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 3, widget); return FALSE; }
gboolean on_entry53_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 3, widget); return FALSE; }
gboolean on_entry54_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 3, widget); return FALSE; }
gboolean on_entry55_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(5, 3, widget); return FALSE; }
gboolean on_entry56_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(6, 3, widget); return FALSE; }
gboolean on_entry57_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(7, 3, widget); return FALSE; }

gboolean on_entry58_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 4, widget); return FALSE; }
gboolean on_entry59_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 4, widget); return FALSE; }
gboolean on_entry60_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 4, widget); return FALSE; }
gboolean on_entry61_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 4, widget); return FALSE; }
gboolean on_entry62_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 4, widget); return FALSE; }
gboolean on_entry63_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(5, 4, widget); return FALSE; }
gboolean on_entry64_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(6, 4, widget); return FALSE; }
gboolean on_entry65_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(7, 4, widget); return FALSE; }

gboolean on_entry66_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 5, widget); return FALSE; }
gboolean on_entry67_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 5, widget); return FALSE; }
gboolean on_entry68_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 5, widget); return FALSE; }
gboolean on_entry69_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 5, widget); return FALSE; }
gboolean on_entry70_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 5, widget); return FALSE; }
gboolean on_entry71_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(5, 5, widget); return FALSE; }
gboolean on_entry72_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(6, 5, widget); return FALSE; }
gboolean on_entry73_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(7, 5, widget); return FALSE; }

gboolean on_entry74_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 6, widget); return FALSE; }
gboolean on_entry75_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 6, widget); return FALSE; }
gboolean on_entry76_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 6, widget); return FALSE; }
gboolean on_entry77_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 6, widget); return FALSE; }
gboolean on_entry78_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 6, widget); return FALSE; }
gboolean on_entry79_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(5, 6, widget); return FALSE; }
gboolean on_entry80_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(6, 6, widget); return FALSE; }
gboolean on_entry81_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(7, 6, widget); return FALSE; }

gboolean on_entry82_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(0, 7, widget); return FALSE; }
gboolean on_entry83_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(1, 7, widget); return FALSE; }
gboolean on_entry84_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(2, 7, widget); return FALSE; }
gboolean on_entry85_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(3, 7, widget); return FALSE; }
gboolean on_entry86_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(4, 7, widget); return FALSE; }
gboolean on_entry87_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(5, 7, widget); return FALSE; }
gboolean on_entry88_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(6, 7, widget); return FALSE; }
gboolean on_entry89_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) { gpk_actor_game(7, 7, widget); return FALSE; }


gboolean on_drawingarea1_button_press_event(GtkWidget *here, GdkEventButton *event, gpointer user_data)
{
	g_print ("Josh\n");
	return FALSE;
}


gboolean on_button69_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
  return FALSE;
}


gboolean on_button70_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
  return FALSE;
}


gboolean on_button71_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
  return FALSE;
}


gboolean on_button72_button_press_event(GtkWidget *here, GdkEventButton *event, gpointer user_data)
{
	gpk_draw_position(here);
	g_print ("Click\n");
	return FALSE;
}

